@@ -23,15 +23,10 @@ sub fail_handler{
 	return $h;
 }
 
-sub throw{
-	shift; # this class
-
+sub croak{
 	if($] < 5.010_000){
 		require MRO::Compat;
 	}
-	else{
-		require mro;
-	}
 
 	my $caller_pkg;
 	my $i = 0;
@@ -48,14 +43,17 @@ sub throw{
 			last;
 		}
 	}
-	die $fail_handler ? $fail_handler->(@_) : do{ require Carp; Carp::longmess(@_) };
+
+	require Carp;
+	local $Carp::CarpLevel = $Carp::CarpLevel + $i;
+	die $fail_handler ? &{$fail_handler} : &Carp::longmess;
 }
 1;
 __END__
 
 =head1 NAME
 
-Data::Util::Error - Deals with errors in Data::Util
+Data::Util::Error - Deals with package-specific errors in Data::Util
 
 =head1 SYNOPSIS
 
@@ -68,4 +66,16 @@ Data::Util::Error - Deals with errors in Data::Util
 		# ...
 	}
 
+=head1 Functions
+
+=over 4
+
+=item Data::Util::Error->fail_handler()
+
+=item Data::Util::Error->fail_handler($handler)
+
+=item Data::Util::Error::croak(@args)
+
+=back
+
 =cut
